@@ -1,38 +1,31 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2007,2008 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2007-2010 -- leonerd@leonerd.org.uk
 
 package Socket::GetAddrInfo;
 
 use strict;
-
-use Exporter;
-use DynaLoader;
+use warnings;
 
 use Carp;
+
 use Scalar::Util qw( dualvar );
 
 my %errstr;
 
 BEGIN {
-   our @ISA = qw( Exporter );
-   our $VERSION = "0.08_7";
+   our $VERSION = "0.18_001";
 
    our @EXPORT = qw(
       getaddrinfo
       getnameinfo
    );
 
-   push @ISA, qw( DynaLoader ); # Must be last so we can pop it if necessary
-
-   if( not $ENV{NO_GETADDRINFO_XS} and eval { __PACKAGE__->DynaLoader::bootstrap( $VERSION ); 1 } ) {
+   if( not $ENV{NO_GETADDRINFO_XS} and eval { require XSLoader; XSLoader::load( __PACKAGE__, $VERSION ); 1 } ) {
       # Do nothing
    }
    else {
-      # Not a DynaLoader any more
-      pop @ISA;
-
       *getaddrinfo = \&fake_getaddrinfo;
       *getnameinfo = \&fake_getnameinfo;
 
@@ -80,12 +73,14 @@ functions.
 
 =head1 SYNOPSIS
 
+ use Socket qw( SOCK_STREAM );
  use Socket::GetAddrInfo qw( :newapi getaddrinfo getnameinfo );
  use IO::Socket;
 
  my $sock;
 
- my ( $err, @res ) = getaddrinfo( "www.google.com", "www" );
+ my %hints = ( socktype => SOCK_STREAM );
+ my ( $err, @res ) = getaddrinfo( "www.google.com", "www", \%hints );
 
  die "Cannot resolve name - $err" if $err;
 
@@ -137,27 +132,30 @@ sub import
    my $class = shift;
    my %symbols = map { $_ => 1 } @_;
 
-   if( delete $symbols{':newapi'} ) {
-      # Caller wants the new API functions - do nothing
-   }
-   else {
-      # Caller wants the Socket6 backward compatible API functions instead
-      delete $symbols{':Socket6api'} or carp <<EOF;
+   my $api;
+   $api = "new"     if delete $symbols{':newapi'};
+   $api = "Socket6" if delete $symbols{':Socket6api'};
+
+   if( $symbols{getaddrinfo} or $symbols{getnameinfo} ) {
+      defined $api or carp <<EOF;
 Importing Socket::GetAddrInfo without ':newapi' or ':Socket6api' tag.
 Defaults to :Socket6api currently but default will change in a future version.
 EOF
+      if( !defined $api or $api eq "Socket6" ) {
+         my $callerpkg = caller;
 
-      my $callerpkg = caller;
-
-      no strict 'refs';
-      *{"${callerpkg}::getaddrinfo"} = \&Socket6_getaddrinfo if delete $symbols{getaddrinfo};
-      *{"${callerpkg}::getnameinfo"} = \&Socket6_getnameinfo if delete $symbols{getnameinfo};
+         no strict 'refs';
+         *{"${callerpkg}::getaddrinfo"} = \&Socket6_getaddrinfo if delete $symbols{getaddrinfo};
+         *{"${callerpkg}::getnameinfo"} = \&Socket6_getnameinfo if delete $symbols{getnameinfo};
+      }
    }
 
    return unless keys %symbols;
 
+   require Exporter;
+
    local $Exporter::ExportLevel = $Exporter::ExportLevel + 1;
-   $class->SUPER::import( keys %symbols );
+   Exporter::import( $class, keys %symbols );
 }
 
 =head1 FUNCTIONS
@@ -177,6 +175,11 @@ neater argument / return results, and error reporting. The C<:Socket6api>
 style is provided as backward-compatibility for code that wants to use
 C<Socket6>.
 
+If neither style is selected, then this module will provide a Socket6-like API
+to be compatible with earlier versions of C<Socket::GetAddrInfo>. This
+behaviour will change in a later version of the module - make sure to always
+specify the required API type.
+
 =cut
 
 =head2 ( $err, @res ) = getaddrinfo( $host, $service, $hints )
@@ -524,17 +527,35 @@ sub fake_getnameinfo
 
 __END__
 
+=head1 BUGS
+
+=over 4
+
+=item *
+
+At the time of writing, there are no test reports from the C<MSWin32> platform
+either PASS or FAIL. I suspect the code will not currently work as it stands
+on that platform, but it should be fairly easy to fix, as C<Socket6> is known
+to work there. Patches welcomed. :)
+
+=back
+
 =head1 SEE ALSO
 
 =over 4
 
 =item *
 
-L<RFC 2553|http://tools.ietf.org/html/rfc2553> - Basic Socket Interface
-Extensions for IPv6
+L<http://tools.ietf.org/html/rfc2553> - Basic Socket Interface Extensions for
+IPv6
 
 =back
 
+=head1 ACKNOWLEDGEMENTS
+
+With thanks to Zefram <zefram@fysh.org> for help with fixing some bugs in the
+XS code.
+
 =head1 AUTHOR
 
-Paul Evans E<lt>leonerd@leonerd.org.ukE<gt>
+Paul Evans <leonerd@leonerd.org.uk>
